from dataclasses import dataclass

from .configuration_data import ConfigurationData
from .collision_data import CollisionData


@dataclass
class Data:
    configuration: ConfigurationData
    collision: CollisionData



from adam import Simulation, ConfigurationsManager
from adam.entities import Configuration, Data


def main():
    sim: Simulation = Simulation()
    initial_data: Data = sim.load_scene()

    configuration_list: list[Configuration] = ConfigurationsManager.load('tests/data/pick_and_place.csv')

    for configuration in configuration_list:
        sim.render()

        data: Data = sim.step(configuration, initial_data.configuration.right_manipulator)

    sim.close()


if __name__ == '__main__':
    main()



from adam import Simulation
from adam.entities import Configuration, Data


def main():
    sim: Simulation = Simulation()
    initial_data: Data = sim.load_scene()

    left_configuration: Configuration = initial_data.configuration.left_manipulator
    right_configuration: Configuration = initial_data.configuration.right_manipulator

    while sim.is_alive:
        left_configuration += Configuration(0.0, 0.0, 0.0, 0.0, 0.1, 0.0)
        right_configuration -= Configuration(0.0, 0.0, 0.0, 0.0, 0.1, 0.0)

        sim.render()
        data: Data = sim.step(left_configuration, right_configuration)

    sim.close()


if __name__ == '__main__':
    main()



import mujoco
import mujoco_viewer
import time
import pkg_resources

from .entities import Configuration, Data
from .use_cases import DataManager


class Simulation:
    def __init__(self) -> None:
        self.is_alive: bool = True
        self.data_manager: DataManager = DataManager()

    def load_scene(self, filename: str | None = None) -> Data:
        if filename is None:
            filename = pkg_resources.resource_filename('adam', 'assets/scene.xml')

        self.model = mujoco.MjModel.from_xml_path(filename)  # type: ignore
        self.data = mujoco.MjData(self.model)  # type: ignore
        self._load_viewer()

        mujoco.mj_resetDataKeyframe(self.model, self.data, 0)  # type: ignore

        self.data_manager.update(self.data)
        return self.data_manager.get()

    def _load_viewer(self) -> None:
        self.viewer = mujoco_viewer.MujocoViewer(self.model, self.data)

        self.set_view((0, 0, 0.5), 135, -20, 3.5)

    def set_view(self, center: tuple[float, float, float], azimuth: float, elevation: float, distance: float) -> None:
        self.viewer.cam.elevation = elevation
        self.viewer.cam.azimuth = azimuth
        self.viewer.cam.distance = distance
        self.viewer.cam.lookat = center

    def extend_collisions(self, collision_dict: dict[int, str]) -> None:
        self.data_manager.collision_detector.collision_dict.update(collision_dict)

    def step(self, left_configuration: Configuration, right_configuration: Configuration) -> Data:
        # Send configuration
        self.data.qpos[:] = (*left_configuration, *right_configuration)

        # Step on the simulation
        mujoco.mj_step(self.model, self.data)  # type: ignore

        # Get process
        self.is_alive = self.viewer.is_alive

        self.data_manager.update(self.data)
        return self.data_manager.get()

    def render(self, *, fps: int = 30) -> None:
        self.viewer.render()
        time.sleep(1/fps)

    def close(self) -> None:
        self.viewer.close()



import numpy as np

from .collision_detector import CollisionDetector
from ..entities import Data, CollisionData, ConfigurationData, Collision, Configuration


class DataManager:
    def __init__(self) -> None:
        self.configurations: np.ndarray = None  # type:ignore
        self.geometry_1_list: np.ndarray = None  # type:ignore
        self.geometry_2_list: np.ndarray = None  # type:ignore

        self.collision_detector: CollisionDetector = CollisionDetector()

    def update(self, data) -> None:
        self.configurations = data.qpos.copy()
        self.geometry_1_list = data.contact.geom1.copy()
        self.geometry_2_list = data.contact.geom2.copy()

    def get(self) -> Data:
        left_manipulator_configuration: Configuration = Configuration(*self.configurations[0:6])
        right_manipulator_configuration: Configuration = Configuration(*self.configurations[6:12])

        left_manipulator_collision: Collision = self.collision_detector.check_left_manipulator(self.geometry_1_list, self.geometry_2_list)
        right_manipulator_collision: Collision = self.collision_detector.check_right_manipulator(self.geometry_1_list, self.geometry_2_list)

        configuration_data: ConfigurationData = ConfigurationData(left_manipulator_configuration, right_manipulator_configuration)
        collision_data: CollisionData = CollisionData(left_manipulator_collision, right_manipulator_collision)

        return Data(configuration_data, collision_data)



import numpy as np

from typing import NamedTuple


class Configuration(NamedTuple):
    q1: float
    q2: float
    q3: float
    q4: float
    q5: float
    q6: float

    def __repr__(self) -> str:
        return f'({self.q1:.4f}, {self.q2:.4f}, {self.q3:.4f}, {self.q4:.4f}, {self.q5:.4f}, {self.q6:.4f})'

    def __add__(self, other):
        return Configuration(self.q1+other.q1,
                             self.q2+other.q2,
                             self.q3+other.q3,
                             self.q4+other.q4,
                             self.q5+other.q5,
                             self.q6+other.q6)

    def __sub__(self, other):
        return Configuration(self.q1-other.q1,
                             self.q2-other.q2,
                             self.q3-other.q3,
                             self.q4-other.q4,
                             self.q5-other.q5,
                             self.q6-other.q6)

    def __mul__(self, scalar: float):
        return Configuration(self.q1*scalar,
                             self.q2*scalar,
                             self.q3*scalar,
                             self.q4*scalar,
                             self.q5*scalar,
                             self.q6*scalar)

    def __rmul__(self, scalar: float):
        return Configuration(self.q1*scalar,
                             self.q2*scalar,
                             self.q3*scalar,
                             self.q4*scalar,
                             self.q5*scalar,
                             self.q6*scalar)

    def __truediv__(self, scalar: float):
        return Configuration(self.q1/scalar,
                             self.q2/scalar,
                             self.q3/scalar,
                             self.q4/scalar,
                             self.q5/scalar,
                             self.q6/scalar)

    def to_degrees(self):
        return self*180/np.pi

    def to_radians(self):
        return self*np.pi/180

    def to_numpy(self) -> np.ndarray:
        return np.array(self)



from typing import NamedTuple

class Collision(NamedTuple):
    collided: bool
    vector: list[bool]
    shoulder: list[str]
    upper_arm: list[str]
    forearm: list[str]
    wrist_1: list[str]
    wrist_2: list[str]
    wrist_3: list[str]
    
    @classmethod
    def empty(cls):
        return Collision(False, [], [], [], [], [], [], [])
    


from .formatter import Formatter


class Logger:
    @staticmethod
    def info(*message, sep: str = '') -> None:
        prefix: str = Formatter.green(Formatter.bold('[INFO]'))
        output_message: str = Formatter.green(sep.join(map(str, message)))
        print(f'{prefix} {output_message}')

    @staticmethod
    def debug(*message, sep: str = '') -> None:
        prefix: str = Formatter.blue(Formatter.bold('[DEBUG]'))
        output_message: str = Formatter.blue(sep.join(map(str, message)))
        print(f'{prefix} {output_message}')

    @staticmethod
    def warning(*message, sep: str = '') -> None:
        prefix: str = Formatter.yellow(Formatter.bold('[WARNING]'))
        output_message: str = Formatter.yellow(sep.join(map(str, message)))
        print(f'{prefix} {output_message}')

    @staticmethod
    def error(*message, sep: str = '') -> None:
        prefix: str = Formatter.red(Formatter.bold('[ERROR]'))
        output_message: str = Formatter.red(sep.join(map(str, message)))
        print(f'{prefix} {output_message}')


from dataclasses import dataclass

from ....core.entities.configuration import Configuration


@dataclass
class ConfigurationData:
    left_manipulator: Configuration
    right_manipulator: Configuration
    


import pandas as pd
import pkg_resources


from ...core.entities import Configuration


class ConfigurationsManager:
    @staticmethod
    def load(filename: str) -> list[Configuration]:
        if filename == 'test':
            filename = pkg_resources.resource_filename('adam', 'features/configurations/data/configurations_test.csv')

        data_frame: pd.DataFrame = pd.read_csv(filename)

        configuration_list: list[Configuration] = [Configuration(*row) for row in data_frame.to_numpy()]

        return configuration_list

    @staticmethod
    def save(filename: str, configuration_list: list[Configuration]) -> None:
        data: dict = {
            'theta_1': [configuration.q1 for configuration in configuration_list],
            'theta_2': [configuration.q2 for configuration in configuration_list],
            'theta_3': [configuration.q3 for configuration in configuration_list],
            'theta_4': [configuration.q4 for configuration in configuration_list],
            'theta_5': [configuration.q5 for configuration in configuration_list],
            'theta_6': [configuration.q6 for configuration in configuration_list],
        }

        data_frame: pd.DataFrame = pd.DataFrame(data)

        data_frame.to_csv(filename, index=False)



import numpy as np

from ..entities import Collision


class CollisionDetector:
    def __init__(self) -> None:
        self.collision_dict: dict[int, str] = {
            2: 'body',
            3: 'body',
            4: 'body',
            10: 'shoulder_left',
            18: 'upperarm_left',
            25: 'forearm_left',
            26: 'forearm_left',
            30: 'wrist_1_left',
            34: 'wrist_2_left',
            40: 'wrist_3_left',
            46: 'shoulder_right',
            54: 'upperarm_right',
            61: 'forearm_right',
            62: 'forearm_right',
            66: 'wrist_1_right',
            70: 'wrist_2_right',
            76: 'wrist_3_right',
        }

        self.ignore_list: list[tuple[str, str]] = [('shoulder_left', 'body'), ('shoulder_right', 'body')]

    def check_left_manipulator(self, geometry_1_id_list: np.ndarray, geometry_2_id_list: np.ndarray) -> Collision:
        if not geometry_1_id_list.size or not geometry_2_id_list.size:
            return Collision.empty()

        geometry_1_list: list[str] = [self.collision_dict.get(geometry_1_id, 'unknown') for geometry_1_id in geometry_1_id_list]
        geometry_2_list: list[str] = [self.collision_dict.get(geometry_2_id, 'unknown') for geometry_2_id in geometry_2_id_list]

        shoulder_collision_list: list[str] = []
        upperarm_collision_list: list[str] = []
        forearm_collision_list: list[str] = []
        wrist_1_collision_list: list[str] = []
        wrist_2_collision_list: list[str] = []
        wrist_3_collision_list: list[str] = []

        for geometry_1, geometry_2 in zip(geometry_1_list, geometry_2_list):
            if (geometry_1, geometry_2) in self.ignore_list:
                continue

            match geometry_1:
                case 'shoulder_left':
                    shoulder_collision_list.append(geometry_2) if geometry_2 not in shoulder_collision_list else None
                case 'upperarm_left':
                    upperarm_collision_list.append(geometry_2) if geometry_2 not in upperarm_collision_list else None
                case 'forearm_left':
                    forearm_collision_list.append(geometry_2) if geometry_2 not in forearm_collision_list else None
                case 'wrist_1_left':
                    wrist_1_collision_list.append(geometry_2) if geometry_2 not in wrist_1_collision_list else None
                case 'wrist_2_left':
                    wrist_2_collision_list.append(geometry_2) if geometry_2 not in wrist_2_collision_list else None
                case 'wrist_3_left':
                    wrist_3_collision_list.append(geometry_2) if geometry_2 not in wrist_3_collision_list else None

        collision_vector: list[bool] = [True if shoulder_collision_list else False,
                                        True if upperarm_collision_list else False,
                                        True if forearm_collision_list else False,
                                        True if wrist_1_collision_list else False,
                                        True if wrist_2_collision_list else False,
                                        True if wrist_3_collision_list else False,]

        return Collision(any(collision_vector),
                         collision_vector,
                         shoulder_collision_list,
                         upperarm_collision_list,
                         forearm_collision_list,
                         wrist_1_collision_list,
                         wrist_2_collision_list,
                         wrist_3_collision_list,
                         )

    def check_right_manipulator(self, geometry_1_id_list: np.ndarray, geometry_2_id_list: np.ndarray) -> Collision:
        if not geometry_1_id_list.size or not geometry_2_id_list.size:
            return Collision.empty()

        geometry_1_list: list[str] = [self.collision_dict.get(geometry_1_id, 'unknown') for geometry_1_id in geometry_1_id_list]
        geometry_2_list: list[str] = [self.collision_dict.get(geometry_2_id, 'unknown') for geometry_2_id in geometry_2_id_list]

        shoulder_collision_list: list[str] = []
        upperarm_collision_list: list[str] = []
        forearm_collision_list: list[str] = []
        wrist_1_collision_list: list[str] = []
        wrist_2_collision_list: list[str] = []
        wrist_3_collision_list: list[str] = []

        for geometry_1, geometry_2 in zip(geometry_1_list, geometry_2_list):
            if (geometry_1, geometry_2) in self.ignore_list:
                continue

            match geometry_1:
                case 'shoulder_right':
                    shoulder_collision_list.append(geometry_2) if geometry_2 not in shoulder_collision_list else None
                case 'upperarm_right':
                    upperarm_collision_list.append(geometry_2) if geometry_2 not in upperarm_collision_list else None
                case 'forearm_right':
                    forearm_collision_list.append(geometry_2) if geometry_2 not in forearm_collision_list else None
                case 'wrist_1_right':
                    wrist_1_collision_list.append(geometry_2) if geometry_2 not in wrist_1_collision_list else None
                case 'wrist_2_right':
                    wrist_2_collision_list.append(geometry_2) if geometry_2 not in wrist_2_collision_list else None
                case 'wrist_3_right':
                    wrist_3_collision_list.append(geometry_2) if geometry_2 not in wrist_3_collision_list else None

        collision_vector: list[bool] = [True if shoulder_collision_list else False,
                                        True if upperarm_collision_list else False,
                                        True if forearm_collision_list else False,
                                        True if wrist_1_collision_list else False,
                                        True if wrist_2_collision_list else False,
                                        True if wrist_3_collision_list else False,]

        return Collision(any(collision_vector),
                         collision_vector,
                         shoulder_collision_list,
                         upperarm_collision_list,
                         forearm_collision_list,
                         wrist_1_collision_list,
                         wrist_2_collision_list,
                         wrist_3_collision_list,
                         )
